@@ -1,5 +1,5 @@
 #
-# Copyright (C) 2008 Red Hat, Inc.
+# Copyright (C) 2008, 2013 Red Hat, Inc.
 # Copyright (C) 2008 Cole Robinson <crobinso@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -23,14 +23,13 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 # pylint: enable=E0611
 
-import copy
 import logging
 
-from virtManager import util
 from virtManager.baseclass import vmmGObjectUI
 from virtManager.asyncjob import vmmAsyncJob
+from virtManager import uiutil
 
-from virtinst import Storage
+from virtinst import StoragePool
 
 PAGE_NAME   = 0
 PAGE_FORMAT = 1
@@ -38,13 +37,10 @@ PAGE_FORMAT = 1
 
 class vmmCreatePool(vmmGObjectUI):
     def __init__(self, conn):
-        vmmGObjectUI.__init__(self,
-                              "vmm-create-pool.ui",
-                              "vmm-create-pool")
+        vmmGObjectUI.__init__(self, "createpool.ui", "vmm-create-pool")
         self.conn = conn
 
         self._pool = None
-        self._pool_class = Storage.StoragePool
 
         self.builder.connect_signals({
             "on_pool_forward_clicked" : self.forward,
@@ -53,41 +49,18 @@ class vmmCreatePool(vmmGObjectUI):
             "on_vmm_create_pool_delete_event" : self.close,
             "on_pool_finish_clicked"  : self.forward,
             "on_pool_pages_change_page" : self.page_changed,
+
             "on_pool_source_button_clicked" : self.browse_source_path,
             "on_pool_target_button_clicked" : self.browse_target_path,
 
             "on_pool_name_activate": self.forward,
             "on_pool_hostname_activate" : self.hostname_changed,
             "on_pool_iqn_chk_toggled": self.iqn_toggled,
-
-            "on_pool_name_focus_in_event": self.update_doc_name,
-
-            "on_pool_type_focus": self.update_doc_type,
-            "on_pool_type_changed": self.update_doc_type,
-
-            "on_pool_format_focus": self.update_doc_format,
-            "on_pool_format_changed": self.update_doc_format,
-
-            "on_pool_target_path_focus_in_event": self.update_doc_target_path,
-            "on_pool_target_path_focus": self.update_doc_target_path,
-            "on_pool_target_path_changed": self.update_doc_target_path,
-
-            "on_pool_source_path_focus_in_event": self.update_doc_source_path,
-            "on_pool_source_path_focus": self.update_doc_source_path,
-            "on_pool_source_path_changed": self.update_doc_source_path,
-
-            "on_pool_hostname_focus_in_event": self.update_doc_hostname,
-            "on_pool_build_focus_in_event": self.update_build_doc,
-
-            "on_pool_iqn_focus_in_event": self.update_doc_iqn,
         })
         self.bind_escape_key_close()
 
-        finish_img = Gtk.Image.new_from_stock(Gtk.STOCK_QUIT,
-                                              Gtk.IconSize.BUTTON)
-        self.widget("pool-finish").set_image(finish_img)
-
         self.set_initial_state()
+        self.set_page(PAGE_NAME)
 
     def show(self, parent):
         logging.debug("Showing new pool wizard")
@@ -107,19 +80,18 @@ class vmmCreatePool(vmmGObjectUI):
     def set_initial_state(self):
         self.widget("pool-pages").set_show_tabs(False)
 
+        blue = Gdk.Color.parse("#0072A8")[1]
+        self.widget("header").modify_bg(Gtk.StateType.NORMAL, blue)
+
         type_list = self.widget("pool-type")
         type_model = Gtk.ListStore(str, str)
         type_list.set_model(type_model)
-        text1 = Gtk.CellRendererText()
-        type_list.pack_start(text1, True)
-        type_list.add_attribute(text1, 'text', 1)
+        uiutil.set_combo_text_column(type_list, 1)
 
         format_list = self.widget("pool-format")
         format_model = Gtk.ListStore(str, str)
         format_list.set_model(format_model)
-        text2 = Gtk.CellRendererText()
-        format_list.pack_start(text2, False)
-        format_list.add_attribute(text2, 'text', 1)
+        uiutil.set_combo_text_column(format_list, 1)
 
         # Target path combo box entry
         target_list = self.widget("pool-target-path")
@@ -128,8 +100,6 @@ class vmmCreatePool(vmmGObjectUI):
         target_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         target_list.set_model(target_model)
         target_list.set_entry_text_column(0)
-        target_list.get_child().connect("focus-in-event",
-                                        self.update_doc_target_path)
 
         # Source path combo box entry
         source_list = self.widget("pool-source-path")
@@ -138,16 +108,9 @@ class vmmCreatePool(vmmGObjectUI):
         source_model.set_sort_column_id(0, Gtk.SortType.ASCENDING)
         source_list.set_model(source_model)
         source_list.set_entry_text_column(0)
-        source_list.get_child().connect("focus-in-event",
-                                        self.update_doc_source_path)
 
         self.populate_pool_type()
 
-        self.widget("pool-info-box1").modify_bg(Gtk.StateType.NORMAL,
-                                                Gdk.Color.parse("grey")[1])
-        self.widget("pool-info-box2").modify_bg(Gtk.StateType.NORMAL,
-                                                Gdk.Color.parse("grey")[1])
-
     def reset_state(self):
         self.widget("pool-pages").set_current_page(0)
         self.widget("pool-forward").show()
@@ -166,6 +129,7 @@ class vmmCreatePool(vmmGObjectUI):
         self.widget("pool-format").set_active(-1)
         self.widget("pool-build").set_sensitive(True)
         self.widget("pool-build").set_active(False)
+        self.widget("pool-details-grid").set_visible(False)
 
 
     def hostname_changed(self, ignore):
@@ -178,11 +142,11 @@ class vmmCreatePool(vmmGObjectUI):
     def populate_pool_type(self):
         model = self.widget("pool-type").get_model()
         model.clear()
-        types = Storage.StoragePool.get_pool_types()
+        types = StoragePool.get_pool_types()
         types.sort()
         for typ in types:
             model.append([typ, "%s: %s" %
-                         (typ, Storage.StoragePool.get_pool_type_desc(typ))])
+                         (typ, StoragePool.get_pool_type_desc(typ))])
 
     def populate_pool_format(self, formats):
         model = self.widget("pool-format").get_model()
@@ -202,24 +166,24 @@ class vmmCreatePool(vmmGObjectUI):
         use_list = source_list
         use_model = source_model
         entry_list = []
-        if self._pool.type == Storage.StoragePool.TYPE_SCSI:
+        if self._pool.type == StoragePool.TYPE_SCSI:
             entry_list = self.list_scsi_adapters()
             use_list = source_list
             use_model = source_model
 
-        elif self._pool.type == Storage.StoragePool.TYPE_LOGICAL:
+        elif self._pool.type == StoragePool.TYPE_LOGICAL:
             pool_list = self.list_pool_sources()
             entry_list = [[p.target_path, p.target_path, p]
                           for p in pool_list]
             use_list = target_list
             use_model = target_model
 
-        elif self._pool.type == Storage.StoragePool.TYPE_DISK:
+        elif self._pool.type == StoragePool.TYPE_DISK:
             entry_list = self.list_disk_devs()
             use_list = source_list
             use_model = source_model
 
-        elif self._pool.type == Storage.StoragePool.TYPE_NETFS:
+        elif self._pool.type == StoragePool.TYPE_NETFS:
             host = self.get_config_host()
             if host:
                 pool_list = self.list_pool_sources(host=host)
@@ -240,12 +204,11 @@ class vmmCreatePool(vmmGObjectUI):
 
         clean_list = []
         for h in host_list:
-            tmppool = copy.copy(self._pool)
             name = "host%s" % h
-
+            tmppool = self._make_stub_pool()
             tmppool.source_path = name
-            entry = [name, name, tmppool]
 
+            entry = [name, name, tmppool]
             if name not in [l[0] for l in clean_list]:
                 clean_list.append(entry)
 
@@ -262,7 +225,7 @@ class vmmCreatePool(vmmGObjectUI):
         devlist.sort()
         clean_list = []
         for dev in devlist:
-            tmppool = copy.copy(self._pool)
+            tmppool = self._make_stub_pool()
             tmppool.source_path = dev
 
             entry = [dev, dev, tmppool]
@@ -272,14 +235,14 @@ class vmmCreatePool(vmmGObjectUI):
         return clean_list
 
     def list_pool_sources(self, host=None):
-        name = self.get_config_name()
         pool_type = self._pool.type
 
         plist = []
         try:
-            plist = Storage.StoragePool.pool_list_from_sources(self.conn.vmm,
-                                                               name, pool_type,
-                                                               host=host)
+            plist = StoragePool.pool_list_from_sources(
+                                                self.conn.get_backend(),
+                                                pool_type,
+                                                host=host)
         except Exception:
             logging.exception("Pool enumeration failed")
 
@@ -287,22 +250,23 @@ class vmmCreatePool(vmmGObjectUI):
 
     def show_options_by_pool(self):
         def show_row(base, do_show):
-            self.widget(base + "-label").set_property("visible", do_show)
-            self.widget(base + "-box").set_property("visible", do_show)
+            widget = self.widget(base + "-label")
+            uiutil.set_grid_row_visible(widget, do_show)
 
-        src     = hasattr(self._pool, "source_path")
-        src_b   = src and not self.conn.is_remote()
-        tgt     = hasattr(self._pool, "target_path")
-        tgt_b   = tgt and not self.conn.is_remote()
-        host    = hasattr(self._pool, "host")
-        fmt     = hasattr(self._pool, "formats")
-        iqn     = hasattr(self._pool, "iqn")
+        src = self._pool.supports_property("source_path")
+        src_b = src and not self.conn.is_remote()
+        src_name = self._pool.type == StoragePool.TYPE_GLUSTER
+        tgt = self._pool.supports_property("target_path")
+        tgt_b = tgt and not self.conn.is_remote()
+        host = self._pool.supports_property("host")
+        fmt = self._pool.supports_property("formats")
+        iqn = self._pool.supports_property("iqn")
         builddef, buildsens = self.get_build_default()
 
         # Source path broswing is meaningless for net pools
-        if self._pool.type in [Storage.StoragePool.TYPE_NETFS,
-                               Storage.StoragePool.TYPE_ISCSI,
-                               Storage.StoragePool.TYPE_SCSI]:
+        if self._pool.type in [StoragePool.TYPE_NETFS,
+                               StoragePool.TYPE_ISCSI,
+                               StoragePool.TYPE_SCSI]:
             src_b = False
 
         show_row("pool-target", tgt)
@@ -311,25 +275,30 @@ class vmmCreatePool(vmmGObjectUI):
         show_row("pool-format", fmt)
         show_row("pool-build", buildsens)
         show_row("pool-iqn", iqn)
+        show_row("pool-source-name", src_name)
+
+        if tgt:
+            self.widget("pool-target-path").get_child().set_text(
+                self._pool.target_path)
 
-        self.widget("pool-target-path").get_child().set_text(self._pool.target_path)
         self.widget("pool-target-button").set_sensitive(tgt_b)
         self.widget("pool-source-button").set_sensitive(src_b)
         self.widget("pool-build").set_active(builddef)
 
+        if src_name:
+            self.widget("pool-source-name").get_child().set_text(
+                self._pool.source_name)
+
         self.widget("pool-format").set_active(-1)
         if fmt:
-            self.populate_pool_format(getattr(self._pool, "formats"))
+            self.populate_pool_format(self._pool.list_formats("formats"))
             self.widget("pool-format").set_active(0)
 
         self.populate_pool_sources()
 
 
     def get_config_type(self):
-        typ = self.widget("pool-type")
-        if typ.get_active_iter() is not None:
-            return typ.get_model().get_value(typ.get_active_iter(), 0)
-        return None
+        return uiutil.get_list_selection(self.widget("pool-type"), 0)
 
     def get_config_name(self):
         return self.widget("pool-name").get_text()
@@ -339,12 +308,9 @@ class vmmCreatePool(vmmGObjectUI):
         if not src.get_sensitive():
             return None
 
-        # If we provide the user with a drop down
-        model = src.get_model()
-        selection = src.get_active()
-        if selection != -1:
-            return model[selection][1]
-
+        ret = uiutil.get_list_selection(src, 1)
+        if ret is not None:
+            return ret
         return src.get_child().get_text()
 
     def get_config_source_path(self):
@@ -352,12 +318,9 @@ class vmmCreatePool(vmmGObjectUI):
         if not src.get_sensitive():
             return None
 
-        # If we provide the user with a drop down
-        model = src.get_model()
-        selection = src.get_active()
-        if selection != -1:
-            return model[selection][1]
-
+        ret = uiutil.get_list_selection(src, 1)
+        if ret is not None:
+            return ret
         return src.get_child().get_text().strip()
 
     def get_config_host(self):
@@ -366,17 +329,18 @@ class vmmCreatePool(vmmGObjectUI):
             return host.get_text().strip()
         return None
 
-    def get_config_format(self):
-        format_combo = self.widget("pool-format")
-        model = format_combo.get_model()
-        if format_combo.get_active_iter() is not None:
-            model = format_combo.get_model()
-            return model.get_value(format_combo.get_active_iter(), 0)
+    def get_config_source_name(self):
+        name = self.widget("pool-source-name")
+        if name.get_sensitive():
+            return name.get_text().strip()
         return None
 
+    def get_config_format(self):
+        return uiutil.get_list_selection(self.widget("pool-format"), 0)
+
     def get_config_iqn(self):
         iqn = self.widget("pool-iqn")
-        if iqn.get_sensitive() and iqn.get_property("visible"):
+        if iqn.get_sensitive() and iqn.get_visible():
             return iqn.get_text().strip()
         return None
 
@@ -384,13 +348,13 @@ class vmmCreatePool(vmmGObjectUI):
         """ Return (default value, whether build option can be changed)"""
         if not self._pool:
             return (False, False)
-        if self._pool.type in [Storage.StoragePool.TYPE_DIR,
-                               Storage.StoragePool.TYPE_FS,
-                               Storage.StoragePool.TYPE_NETFS]:
+        if self._pool.type in [StoragePool.TYPE_DIR,
+                               StoragePool.TYPE_FS,
+                               StoragePool.TYPE_NETFS]:
             # Building for these simply entails creating a directory
             return (True, False)
-        elif self._pool.type in [Storage.StoragePool.TYPE_LOGICAL,
-                                 Storage.StoragePool.TYPE_DISK]:
+        elif self._pool.type in [StoragePool.TYPE_LOGICAL,
+                                 StoragePool.TYPE_DISK]:
             # This is a dangerous operation, anything (False, True)
             # should be assumed to be one.
             return (False, True)
@@ -415,70 +379,77 @@ class vmmCreatePool(vmmGObjectUI):
     def forward(self, ignore=None):
         notebook = self.widget("pool-pages")
         try:
-            if(self.validate(notebook.get_current_page()) is not True):
+            if self.validate(notebook.get_current_page()) is not True:
                 return
             if notebook.get_current_page() == PAGE_FORMAT:
                 self.finish()
             else:
-                self.widget("pool-forward").grab_focus()
                 notebook.next_page()
         except Exception, e:
             self.err.show_err(_("Uncaught error validating input: %s") % str(e))
             return
 
     def back(self, ignore=None):
-        self.widget("pool-finish").hide()
-        self.widget("pool-forward").show()
         self.widget("pool-pages").prev_page()
 
-    def finish(self):
-        self.topwin.set_sensitive(False)
-        self.topwin.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.WATCH))
-        build = self.widget("pool-build").get_active()
-
-        progWin = vmmAsyncJob(self._async_pool_create, [build],
-                              _("Creating storage pool..."),
-                              _("Creating the storage pool may take a "
-                                "while..."),
-                              self.topwin)
-        error, details = progWin.run()
-
+    def _finish_cb(self, error, details):
         self.topwin.set_sensitive(True)
-        self.topwin.get_window().set_cursor(Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
+        self.topwin.get_window().set_cursor(
+            Gdk.Cursor.new(Gdk.CursorType.TOP_LEFT_ARROW))
 
         if error:
             error = _("Error creating pool: %s") % error
             self.err.show_err(error,
                               details=details)
         else:
+            self.conn.schedule_priority_tick(pollpool=True)
             self.close()
 
-    def _async_pool_create(self, asyncjob, build):
-        newconn = None
+    def finish(self):
+        self.topwin.set_sensitive(False)
+        self.topwin.get_window().set_cursor(
+            Gdk.Cursor.new(Gdk.CursorType.WATCH))
+        build = self.widget("pool-build").get_active()
 
-        # Open a seperate connection to install on since this is async
-        newconn = util.dup_lib_conn(self._pool.conn)
+        progWin = vmmAsyncJob(self._async_pool_create, [build],
+                              self._finish_cb, [],
+                              _("Creating storage pool..."),
+                              _("Creating the storage pool may take a "
+                                "while..."),
+                              self.topwin)
+        progWin.run()
+
+    def _async_pool_create(self, asyncjob, build):
         meter = asyncjob.get_meter()
-        self._pool.conn = newconn
 
         logging.debug("Starting backround pool creation.")
         poolobj = self._pool.install(create=True, meter=meter, build=build)
         poolobj.setAutostart(True)
         logging.debug("Pool creation succeeded")
 
-    def page_changed(self, notebook_ignore, page_ignore, page_number):
-        if page_number == PAGE_NAME:
-            self.widget("pool-back").set_sensitive(False)
-            self.widget("pool-finish").hide()
-            self.widget("pool-forward").show()
-            self.widget("pool-forward").grab_focus()
-        elif page_number == PAGE_FORMAT:
-            self.widget("pool-back").set_sensitive(True)
-            self.widget("pool-finish").show()
-            self.widget("pool-finish").grab_focus()
-            self.widget("pool-forward").hide()
+    def set_page(self, page_number):
+        # Update page number
+        page_lbl = ("<span color='#59B0E2'>%s</span>" %
+                    _("Step %(current_page)d of %(max_page)d") %
+                    {'current_page': page_number + 1,
+                     'max_page': PAGE_FORMAT + 1})
+        self.widget("header-pagenum").set_markup(page_lbl)
+
+        isfirst = (page_number == PAGE_NAME)
+        islast = (page_number == PAGE_FORMAT)
+
+        self.widget("pool-back").set_sensitive(not isfirst)
+        self.widget("pool-finish").set_visible(islast)
+        self.widget("pool-forward").set_visible(not islast)
+        self.widget(islast and "pool-finish" or "pool-forward").grab_focus()
+
+        self.widget("pool-details-grid").set_visible(islast)
+        if islast:
             self.show_options_by_pool()
 
+    def page_changed(self, notebook_ignore, page_ignore, page_number):
+        self.set_page(page_number)
+
     def get_pool_to_validate(self):
         """
         Return a pool instance to use for parameter assignment validation.
@@ -488,114 +459,79 @@ class vmmCreatePool(vmmGObjectUI):
         source_list = self.widget("pool-source-path")
         target_list = self.widget("pool-target-path")
 
-        pool = copy.copy(self._pool)
-
-        if source_list.get_active() != -1:
-            pool = source_list.get_model()[source_list.get_active()][2]
-        elif target_list.get_active() != -1:
-            pool = target_list.get_model()[target_list.get_active()][2]
+        pool = uiutil.get_list_selection(source_list, 2)
+        if pool is None:
+            pool = uiutil.get_list_selection(target_list, 2)
 
         return pool
 
+    def _make_stub_pool(self):
+        pool = StoragePool(self.conn.get_backend())
+        pool.type = self.get_config_type()
+        return pool
+
+    def _validate_page_name(self, usepool=None):
+        try:
+            if usepool:
+                self._pool = usepool
+            else:
+                self._pool = self._make_stub_pool()
+            self._pool.name = self.get_config_name()
+        except ValueError, e:
+            return self.err.val_err(_("Pool Parameter Error"), e)
+
+        return True
+
+    def _validate_page_format(self):
+        target = self.get_config_target_path()
+        host = self.get_config_host()
+        source = self.get_config_source_path()
+        fmt = self.get_config_format()
+        iqn = self.get_config_iqn()
+        source_name = self.get_config_source_name()
+
+        if not self._validate_page_name(self.get_pool_to_validate()):
+            return
+
+        try:
+            self._pool.target_path = target
+            if host:
+                self._pool.host = host
+            if source:
+                self._pool.source_path = source
+            if fmt:
+                self._pool.format = fmt
+            if iqn:
+                self._pool.iqn = iqn
+            if source_name:
+                self._pool.source_name = source_name
+
+            self._pool.validate()
+        except ValueError, e:
+            return self.err.val_err(_("Pool Parameter Error"), e)
+
+        buildval = self.widget("pool-build").get_active()
+        buildsen = (self.widget("pool-build").get_sensitive() and
+                    self.widget("pool-build").get_visible())
+        if buildsen and buildval:
+            ret = self.err.yes_no(_("Building a pool of this type will "
+                                    "format the source device. Are you "
+                                    "sure you want to 'build' this pool?"))
+            if not ret:
+                return ret
+
+        return True
+
     def validate(self, page):
         if page == PAGE_NAME:
-            typ  = self.get_config_type()
-            name = self.get_config_name()
-            conn = self.conn.vmm
-
-            try:
-                self._pool_class = Storage.StoragePool.get_pool_class(typ)
-                self._pool = self._pool_class(name=name, conn=conn)
-            except ValueError, e:
-                return self.err.val_err(_("Pool Parameter Error"), e)
-
-            return True
-
+            return self._validate_page_name()
         elif page == PAGE_FORMAT:
-            target  = self.get_config_target_path()
-            host    = self.get_config_host()
-            source  = self.get_config_source_path()
-            fmt     = self.get_config_format()
-            iqn     = self.get_config_iqn()
-
-            tmppool = self.get_pool_to_validate()
-            try:
-                tmppool.target_path = target
-                if host:
-                    tmppool.host = host
-                if source:
-                    tmppool.source_path = source
-                if fmt:
-                    tmppool.format = fmt
-                if iqn:
-                    tmppool.iqn = iqn
-
-                tmppool.get_xml_config()
-            except ValueError, e:
-                return self.err.val_err(_("Pool Parameter Error"), e)
-
-            buildval = self.widget("pool-build").get_active()
-            buildsen = (self.widget("pool-build").get_sensitive() and
-                        self.widget("pool-build-box").get_property("visible"))
-            if buildsen and buildval:
-                ret = self.err.yes_no(_("Building a pool of this type will "
-                                        "format the source device. Are you "
-                                        "sure you want to 'build' this pool?"))
-                if not ret:
-                    return ret
-
-            self._pool = tmppool
-            return True
-
-    def _update_doc(self, param, infobox):
-        doc = self._build_doc_str(param)
-        self.widget(infobox).set_markup(doc)
-
-    def update_doc_name(self, *ignore):
-        self._update_doc("name", "pool-info1")
-    def update_doc_type(self, *ignore):
-        self._update_doc("type", "pool-info1")
-    def update_doc_target_path(self, *ignore):
-        self._update_doc("target_path", "pool-info2")
-    def update_doc_source_path(self, *ignore):
-        self._update_doc("source_path", "pool-info2")
-    def update_doc_hostname(self, *ignore):
-        self._update_doc("host", "pool-info2")
-    def update_doc_format(self, *ignore):
-        self._update_doc("format", "pool-info2")
-    def update_doc_iqn(self, *ignore):
-        self._update_doc("iqn", "pool-info2")
-
-    def update_build_doc(self, *ignore):
-        doc = ""
-        docstr = ""
-        if self._pool.type == Storage.StoragePool.TYPE_DISK:
-            docstr = _("Format the source device.")
-        elif self._pool.type == Storage.StoragePool.TYPE_LOGICAL:
-            docstr = _("Create a logical volume group from the source device.")
-
-        if docstr:
-            doc = self._build_doc_str("build", docstr)
-        self.widget("pool-info2").set_markup(doc)
-
-    def _build_doc_str(self, param, docstr=None):
-        doc = ""
-        doctmpl = "<i><u>%s</u>: %s</i>"
-        prettyname = param.replace("_", " ").capitalize()
-
-        if docstr:
-            doc = doctmpl % (prettyname, docstr)
-        elif hasattr(self._pool_class, param):
-            doc = doctmpl % (prettyname,
-                             getattr(self._pool_class, param).__doc__)
-
-        return doc
+            return self._validate_page_format()
 
     def _browse_file(self, dialog_name, startfolder=None, foldermode=False):
         mode = Gtk.FileChooserAction.OPEN
         if foldermode:
             mode = Gtk.FileChooserAction.SELECT_FOLDER
 
-        return util.browse_local(self.topwin, dialog_name, self.conn,
-                                 dialog_type=mode,
-                                 start_folder=startfolder)
+        return self.err.browse_local(self.conn, dialog_name,
+            dialog_type=mode, start_folder=startfolder)
